%% Mechanical impedance a piston attached to a duct with a rigid end
% This script is used to calculate the mechanical impedance of a piston
% attached to the end of a duct, where the other end of the duct is
% terminated with a rigid end-cap.
%
%% Introduction
% This script will plot the driving mechanical impedance of a piston 
% on the end of a duct where the far end of the duct has a rigid end cap. 
%
% It can also be used to plot the driving mechanical impedance when the
% fluid inside the duct has absorption.
%
% This script can be used to generate 
% [Fig 10.5.1, p278 Kinsler et al. (1999)]
% that shows the input mechanical impedance for an air-filled rigidly
% terminated pipe of 1m length and 1cm radius, where the fluid has some
% small absorption.
%
% References for this theory are:
%
% * Lawrence E. Kinsler, Austin R. Frey, Alan B. Coppens, James V. Sanders,
%   (1999), "Fundamentals of Acoustics", 4th edition, Wiley, 
%   ISBN 0471847895.
% * Frank Fahy and Paolo Gardonio (2007),
%   "Sound and Structural Vibration: radiation transmission and response",
%   Second edition, Academic Press, ISBN 0123736331.
%
% Keywords: 
% pipe ; duct ; tube ; piston ; radiation ; flanged ; baffle
%
% Carl Howard 30 December 2012

%close all; clear all

%% Properties of the duct
% Duct diameter $D=2a$, where $a$ is the radius of the duct.
duct_diam=0.1;                    % [m] duct diameter

% To generate [Fig 10.5.1, p278 Kinsler et al. (1999)],
% the radius of the duct is 1cm, so define |duct_diam=0.01*2;|
%duct_diam=0.01*2;                 % [m] duct diameter

%%%
% Cross sectional area of circular duct $S=\pi(D/2)^2$
S_duct_area=pi/4*duct_diam^2;     % [m]

%%%
% Length of duct $L$
L_duct_length=3.0;                   % [m] Length of the duct

% To generate [Fig 10.5.1, p278 Kinsler et al. (1999)],
% the length of the duct as 1m, so define |L_duct_length=1.0;|
%L_duct_length=1.0;                   % [m] Length of the duct


%%%
% Force on the piston $F_z$
F_z     =   1e-3;                   % [N]

%% Properties of the fluid
c_speed_sound=344;      % [m/s] Speed of sound in air at 20 degree c 
rho_density=1.21;       % [kg/m^3] Density of air 

%% Analysis frequency range
% To generate [Fig 10.5.1, p278 Kinsler et al. (1999)],
% the x-axis range is $kL=0 \cdots 3.5 \pi$, where $L=1.0$m.
%
% [Note that the label on the x-axis of the figure is incorrect.]
%
% Hence the frequency range is
%
% $$ kL = \frac{2 \pi f_{\textrm{max}}}{c} L = 3.5 \pi$$
%
% $$ f_{\textrm{max}} = 3.5 \pi \, \frac{c}{2 \pi L} = 
% \frac{3.5 \pi \times 343}{2 \pi \times 1.0} =191\textrm{Hz}$$
%
f_max = 3.5*c_speed_sound/(2*L_duct_length);

freq=[0:0.01:f_max];               % [Hz] Frequency of analysis

%%%
% If the theoretical analysis is to be compared with 
% finite element results, it is more convenient to define
% the frequency range directly in Hertz.

% freq=[0:0.1:200];               % [Hz] Frequency of analysis

%%% 
% Wavelength $\lambda$
lambda = c_speed_sound ./ freq;

%%%
% Wave number $k=\omega/c=(2 \pi f) / c$
k_wave_no=(2*pi*freq)/c_speed_sound;

%%%
% A parameter that is used numerous times is $kL$
kL      = k_wave_no*L_duct_length;

%% Caclulate the mechanical impedance for loss-less duct
% The input mechanical impedance for the piston driving at the end 
% of the duct is 
%
% [Eq. (10.2.4), p273 Kinsler et al. (1999)]
%
% $$ Z_{m0} = (\rho_0 c S) \, 
%        \frac{ \frac{Z_{mL}}{(\rho_0 c S)} + j \tan (kL) }{
%               1 + j \frac{Z_{mL}}{(\rho_0 c S)} \tan (kL)}    $$
%
% For a rigid end cap at $x=L$, the impedance at the end is infinite
% 
% $$ \frac{Z_{mL}}{\rho_0 c S} \rightarrow \infty $$ 
%
% and hence the driving mechanical impedance is
%
% [Eq. (10.2.7), p273 Kinsler et al. (1999)]
%
% $$ Z_{m0} = (\rho_0 c S) \, 
%         \left[
%           -j \cot (kL)
%         \right]      $$
%
% Note, do not confuse _mechanical_ impedance with _acoustic_ impedance.
Z_m0 =     (rho_density*c_speed_sound*S_duct_area) * ...
           ( -1i*cot(kL) );

    
%% Plot the mechanical impedance

figure
set(gca,'fontsize',16);
plot1h=plot(kL/pi,real(Z_m0)/(rho_density*c_speed_sound*S_duct_area),  ...
            kL/pi,imag(Z_m0)/(rho_density*c_speed_sound*S_duct_area),'--');

set(plot1h(:),'linewidth',2);
xlabel('kL/\pi')
ylabel('Normalised Impedance')
leg1h=legend('Theory: Real','Theory: Imaginary','location','southeast');
title('Mechanical impedance of a piston on the end of a duct');
grid

axis([0 3.5 -20 20])

%% Calculate the mechanical impedance for a 'lossy' duct
% This theory is from [Section 10.5, p277 Kinsler et al. (1999)]
%
% The mechanical impedance of the piston when there is absorption in the
% fluid and duct walls is given by
% [Eq. (10.5.4), p277 Kinsler et al. (1999)]
%
% $$ Z_{m0} = (\rho_0 c S) \, 
%        \frac{\alpha L - j \cos(kL) \sin(kL)}{
%                \sin^2(kL) + (\alpha L)^2 \cos^2(kL) }     $$
%
% Note that the value of $\alpha$ for the loss factor in the propagation
% constant $\bf{k}=k - j \alpha$, is not defined in the text, but mentions
% a "small value".
alpha=0.01;

%%%
% The mechanical impedance is
Z_m0_2 =   (rho_density*c_speed_sound*S_duct_area) * ...     
        (alpha*L_duct_length - 1i*cos(kL).*sin(kL)) ./  ...
        (sin(kL).^2 + (alpha*L_duct_length)^2 * cos(kL).^2);

%%%
% If the loss factor $\alpha=0$ then the real part of the impedance (the
% resistance) is zero, and the imagnary part (reactance) is proportional to
% $\cot(kL)$ [p278 Kinsler et al. (1999)]
    

%% Plot the impedance for the 'lossy' duct
% This script can be used to generate 
% [Fig 10.5.1, p278 Kinsler et al. (1999)]
%
% Note that there is an error in the text book with the axes in Fig 10.5.1
% which shows that the x-axis is $kL$ when it should be $kL/\pi$, which can
% be confirmed by reading the discussion, and examination of the equation
% Eq.(10.5.2). 
%

figure
set(gca,'fontsize',16);
plot2h=plot(kL/pi,real(Z_m0_2)/(rho_density*c_speed_sound*S_duct_area), ...
            kL/pi,imag(Z_m0_2)/(rho_density*c_speed_sound*S_duct_area), ...
                    '--');

set(plot2h(:),'linewidth',2);
xlabel('kL/\pi')
ylabel('Normalised Impedance')
leg2h=legend('Theory: Real','Theory: Imaginary','location','southeast');
title('Mechanical impedance of a piston on the end of a duct');
grid

axis([0 3.5 -15 15])


%% Calculate power delivered into the duct
% The mechanical power done by the piston is given by
%
% [Eq. (2.4), p77 Fahy and Gardonio (2007)]
%
% $$ \textrm{Power} = \frac{1}{2} 
%           |\tilde{F}|^2 \, \textrm{Re}( \frac{1}{ Z_{m0} } ) $$
%
% Note there is a mistake in Kinsler et al. (1999) with the equation for
% power on p278.

power = F_z^2 /2 * real(ones(size(Z_m0_2))./Z_m0_2);

%% Plot the mechanical power

figure
set(gca,'fontsize',16);
plot3h=semilogy(kL/pi,power);
set(plot3h(:),'linewidth',2);
xlabel('kL/\pi')
ylabel('Power [Watts]')
leg3h=legend('Theory','location','southeast');
title('Mechanical power used by the piston')
grid
    