%% Impedance Tube with MPP Termination
% This file contains the analysis of an impedance tube
% which reads in data from Ansys.  The APDL script titled
% "code_ansys_impedance_sheet.txt" should be in run in Ansys APDL prior to running
% this file.
%
% Note: If you click on File > Save and Publish to HTML you will have a
% nice record of this tutorial and its outputs.
%
% References:
% [1] A.F. Seybert and D.F. Ross. Experimental determination of acoustic properties
% using a two-microphone random-excitation technique. Journal of Acoustical
% Society of America, 61(5):13621370, 1977.
%
% [2] J.Y. Chung and D.A. Blaser. Transfer function method of measuring in-duct
% acoustic properties. i. theory. Journal of Acoustical Society of America, 68
% (3):907913, 1980.
%
% [3] G. Mller and M. Mser, editors. Handbook of Engineering Acoustics. Springer,
% 2013.
%
% [4]  Maa...
%
% Author: BSC
% Date: 6/6/2013
%
% Revision History
% Date      Change 
%

close all
clear all
clc


%% System sketch
%     ------------------------------
%    |                          |   |
%    |      Duct            MPP |   | Termination Impedance 
%    |                          |   |
%     ------------------------------
%  Source                        
%



%% System parameters
% This cell is used to define all the system parameters
%Duct geometry
L = 1.0                                 % [m] Length of duct
X1 = 0.5                                % [m] Distance of Mic 1 to impedance
X2 = 0.6                                % [m] Distance of Mic 2 to impedance
D = 0.1                                 % [m] Depth of backing cavity 

% Air properties
c_0 = 343.24                            % [m/s] Speed of sound in air
rho_0 = 1.2041                          % [kg/m^3] Density of air
Viscosity = 1.84*10^(-5)                % [N.s/m^2] Dynamic viscosity of air

real_Z = rho_0*c_0                          % [N.s/m^3] Resistance of the material 
imag_Z = -rho_0*c_0                         % [N.s/m^3] Reactance of the material 

%% MPP Properties
d_mpp = 0.001                           % [m] Hole diameter
t_mpp = 0.010                           % [m] Panel thickness
P_mpp = 10/100                          % Perforation ratio


%% Load in Ansys Harmonic Analysis Results
% This cell is used to read in the results from the Ansys harmonic analysis
% Scalar parameters used by Ansys for the analysis
load ('../ansys/apdl/impedance_surf153_mpp_0.txt')

% Microphone 1
load ('../ansys/apdl/impedance_surf153_mpp_1.txt')
f_ansys1 = impedance_surf153_mpp_1(:,1);             % Frequency vector
xfer_ansys1 = impedance_surf153_mpp_1(:,2) + j*impedance_surf153_mpp_1(:,3);   % Complex FRF
mag_ansys1 = abs(xfer_ansys1);            % Magnitude vector
phi_ansys1 = 180/pi*phase(xfer_ansys1);   % Phase vector 

% Microphone 2
load ('../ansys/apdl/impedance_surf153_mpp_2.txt')
f_ansys2 = impedance_surf153_mpp_2(:,1);             % Frequency vector
xfer_ansys2 = impedance_surf153_mpp_2(:,2) + j*impedance_surf153_mpp_2(:,3);   % Complex FRF
mag_ansys2 = abs(xfer_ansys2);            % Magnitude vector
phi_ansys2 = 180/pi*phase(xfer_ansys2);   % Phase vector 


% Microphone 3 - Pressure
load ('../ansys/apdl/impedance_surf153_mpp_3.txt')
f_ansys3 = impedance_surf153_mpp_3(:,1);             % Frequency vector
xfer_ansys3 = impedance_surf153_mpp_3(:,2) + j*impedance_surf153_mpp_3(:,3);   % Complex FRF
mag_ansys3 = abs(xfer_ansys3);            % Magnitude vector
phi_ansys3 = 180/pi*phase(xfer_ansys3);   % Phase vector 

% Microphone 3 - Velocity
load ('../ansys/apdl/impedance_surf153_mpp_4.txt')
f_ansys4 = impedance_surf153_mpp_4(:,1);             % Frequency vector
xfer_ansys4 = impedance_surf153_mpp_4(:,2) + j*impedance_surf153_mpp_4(:,3);   % Complex FRF
%xfer_ansys4 = impedance_4(:,2).*exp(j*impedance_4(:,3)/180*pi);   % Complex FRF
mag_ansys4 = abs(xfer_ansys4);            % Magnitude vector
phi_ansys4 = 180/pi*phase(xfer_ansys4);   % Phase vector 

% Microphone 3 - Displacement
load ('../ansys/apdl/impedance_surf153_mpp_5.txt')
f_ansys5 = impedance_surf153_mpp_5(:,1);             % Frequency vector
xfer_ansys5 = impedance_surf153_mpp_5(:,2) + j*impedance_surf153_mpp_5(:,3);   % Complex FRF
%xfer_ansys4 = impedance_4(:,2).*exp(j*impedance_4(:,3)/180*pi);   % Complex FRF
mag_ansys5 = abs(xfer_ansys5);            % Magnitude vector
phi_ansys5 = 180/pi*phase(xfer_ansys5);   % Phase vector 




%% Process Ansys Results
% This cell processes the Ansys results using the formulae defined below.
%
% The complex reflection coefficient is given by
%
% $$r = e^{2 j k X2}\frac{ H12-e^{-j k d} }{ e^{j k d} -H12} $$
% 
% The normal absorption coefficient is given by
%
% $$\alpha = 1-|r|^2 $$
%
% The surface impedance at the absorbing material is given by
%
% $$ Z = \frac{1+r}{1-r} $$
%

H12_ansys = (xfer_ansys2./xfer_ansys1);     % Transfer function from Mic 1 to Mic 2
d = X2-X1;                          % Distance between mics
k = 2*pi*f_ansys1/c_0;                % Acoustic wavenumber


% Complex reflection coeff
r_ansys = exp(2*j*k*X2).*(( H12_ansys-exp(-j*k*d) )./( exp(j*k*d) -H12_ansys ) )
r_abs = abs(r_ansys)                % Magnitude of the reflection coefficient
alpha_ansys = 1-abs(r_ansys).^2     % Normal absorption coefficient

% Calculate the surface impedance at the absorbing material
Zs_ansys = (1+r_ansys)./(1-r_ansys)


% Directly estimate the impedance ratio using the pressure and velocity at
% the surface
Zs_ansys2 = (xfer_ansys3./xfer_ansys4)/rho_0/c_0    % Normalised impedance
r_ansys2 = (Zs_ansys2-1)./(Zs_ansys2+1) % Reflection coefficient
alpha_ansys2 = 1-abs(r_ansys2).^2       % Normal absorption coefficient

% Directly estimate the impedance ratio using the pressure and displacement at
% the surface
Zs_ansys3 = (xfer_ansys3./(j*k.*xfer_ansys5))/rho_0/c_0^2    % Normalised impedance
r_ansys3 = (Zs_ansys3-1)./(Zs_ansys3+1) % Reflection coefficient
alpha_ansys3 = 1-abs(r_ansys3).^2       % Normal absorption coefficient



%% Theoretical Parameters
% Define the range of frequencies to solve for
f = linspace(50,1000,1001);             % [Hz] Frequency range
omega = 2*pi*f;                         % [rad/s] Frequency range
rad2deg = 180/pi;                       % Radian to degree conversion
k = omega/c_0;                            % Wavenumber vector
s = j*omega;                            % Laplace variable


%k_mpp = d_mpp*sqrt(f/10)*1000;
k_mpp = d_mpp*sqrt(omega*rho_0/(4*Viscosity));

kr_mpp = (1+k_mpp.^2/32).^(1/2) + sqrt(2)/8*k_mpp*d_mpp/t_mpp;
%rmpp = 0.147*t*krmpp(i)/p/(d^2); %+2*sqrt(2*2*pi*f(i)*1.89*10^(-5)*1.21)
r_mpp = (32*Viscosity)/(rho_0*c_0)*(t_mpp/P_mpp/(d_mpp^2))*kr_mpp; % Real part. Maa Eq. (5a)
km_mpp = 1 + (9+k_mpp.^2/2).^(-1/2) + 0.85*d_mpp/t_mpp;
%m_mpp = 0.294*10^(-3)*t_mpp*km_mpp/P_mpp;
m_mpp = t_mpp/(P_mpp*c_0)*km_mpp;			 	% Maa Eq. (5b)
%zt=rmpp+1j.*mmpp.*Wt-1j.*cot(Wt.*D./c);
%zt=(1500+1j.*0.485.*Wt)./(p0*c0)-1j.*cot(Wt.*D./c);

% Directly calculate parameters
real_Z = r_mpp*rho_0*c_0;
imag_Z = omega.*m_mpp*rho_0*c_0;
Zs = (real_Z + j *(imag_Z-rho_0*c_0*cot(2*pi*f*D/c_0)))/rho_0/c_0;     % Complex impedance
r_2 = (Zs-1)./(Zs+1);                 % Complex reflection coefficient
alpha_2  = 1 - abs(r_2).^2;           % Sound absorption coefficient

% Determine the resonance frequency
f_index = min(find(( imag_Z/rho_0/c_0-cot(omega*D/c_0) )>=0))+1;
f_0 = f(f_index)
% Peak absorption
alpha_0 = 4*real(Zs(f_index))./(1+real(Zs(f_index))).^2


%% Harmonic Response in the Duct
% Calculate the pressures at a number of locations in the duct to allow
% comparison with the Ansys results.
% 
% The pressure at a point in the duct (x metres from an impedance with 
% reflection coefficient r) is defined as the sum of forward and backwards
% travelling waves and is given by
%
% $$ p(x) = P_0(e^{jkx} + re^{-jkx}) $$
% 


% Calculate transfer function between mics
p1 = exp(j*k*X1)+r_2.*exp(-j*k*X1);      % Pressure at microphone 1     
p2 = exp(j*k*X2)+r_2.*exp(-j*k*X2);      % Pressure at microphone 2

H12 = (p1./p2);                           % Transfer function from p2 to p1




%% Plot the transfer function
figure(1)
set(gcf,'Name','Transfer function from mic 2 to mic 1')
subplot(2,1,1)                          % Bode magnitude
semilogy(f_ansys1, abs(H12_ansys),...       % Ansys magnitude
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, abs(H12), ...                   % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 100, 0, 5E-04])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Magnitude')
grid


subplot(2,1,2)                          % Bode phase
plot(f_ansys1, angle(H12_ansys)*180/pi,...         % Ansys phase
    '.k', 'MarkerSize',20)
hold on
plot(f,angle(H12)*180/pi, ...                     % Matlab phase
    'Color',[.5 .5 .5], 'LineWidth',2)
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 100,-180,0])
xlabel('Frequency (Hz)')
ylabel('Phase, \phi (degrees)')
legend('ANSYS','MATLAB','Location','Best') 
grid

% Save the plot to a file
print -depsc 'transfer_function_MPP.eps'



%% Plot the impedance
figure(2)
set(gcf,'Name','Termination Impedance')


subplot(2,1,1)                          % Real
plot(f_ansys1, real(Zs_ansys),...       % Ansys real impendance
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, real(Zs), ...                    % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 1000, 0.999, 1.001])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Real')
grid


subplot(2,1,2)                          % Imaginary
plot(f_ansys1, imag(Zs_ansys),...       % Ansys imag impedance
    '.k', 'MarkerSize',20)
hold on
plot(f,imag(Zs), ...      % Matlab imag impedance
    'Color',[.5 .5 .5], 'LineWidth',2)
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 1000,-1.001,-0.999])
xlabel('Frequency (Hz)')
ylabel('Imaginary')
legend('ANSYS','MATLAB','Location','Best') 
grid

% Save the plot to a file
print -depsc 'impedance_MPP.eps'



%% Plot the absorption coefficient
figure(3)
set(gcf,'Name','Absorption Coefficient')
plot(f_ansys1, alpha_ansys,...       % Ansys absorption coefficient
    '.k', 'MarkerSize',20)  
hold on                                 % Hold to overlay
plot(f, alpha_2, ...                    % Matlab magnitude
    'Color',[.5 .5 .5], 'LineWidth',2)  % Set to grey and width 2
plot(f_0, alpha_0,...       % Ansys real impendance
    '*k', 'MarkerSize',10)
hold off
set(gca, 'FontSize', 14)                % Increase font for printing
%axis([0, 1000, 0.999, 1.001])                % Set axis
xlabel('Frequency (Hz)')
ylabel('Absorption Coefficient')
grid
legend('ANSYS','MATLAB','Location','Best') 

% Save the plot to a file
print -depsc 'absorption_coefficient_MPP.eps'



%% Tabulate scalar parameters
% Locate correct index
f_ind = find(f==100);
disp('Comparison of parameters at 100Hz')
disp(['Parameter        ','Matlab   ','Ansys    ','% Diff'])
disp(['Ref. Coeff.    ',num2str((r_2)),'  ',num2str((r_ansys(1))),'  '])
disp(['Mag Ref. Coeff.  ',num2str(abs(r_2)),'  ',num2str(abs(r_ansys(1))),'  ', ...
    num2str((abs(r_ansys(1))-abs(r_2))/abs(r_2)*100)])
disp(['Sound Ab. Coeff. ',num2str(alpha_2),'  ',num2str(alpha_ansys(1)),'  ', ...
    num2str((alpha_ansys(1)-alpha_2)/alpha_2*100)])
disp(['Real Imp. Ratio  ',num2str(real(Zs)),'  ',num2str(real(Zs_ansys(1))),'  ', ...
    num2str((real(Zs_ansys(1))-real(Zs))/real(Zs)*100)])
disp(['Imag Imp. Ratio  ',num2str(imag(Zs)),'  ',num2str(imag(Zs_ansys(1))),'  ', ...
    num2str((imag(Zs_ansys(1))-imag(Zs))/imag(Zs)*100)])

