%% Lined Duct code
% This file contains the analysis of the attenuation along a lined duct.
% This is is done for both a locally reacting liner as well as a bulk
% reacting liner.
% 
% Calculations are based on theory as well as imported results from both
% Ansys Classic and Ansys Workbench.
%
% Note: If you click on File > Save and Publish to HTML you will have a
% nice record of this tutorial and its outputs.
%
% References
% 1. Beranek, L.L., ed., Noise and Vibration Control,
%    2nd edn., Institute of Noise Control Engineering, 1988.
% 2. C. Wassilieff, "Experimental Verification Of Duct Attenuation Models 
%    With Bulk Reacting Linings", Journal of Sound and Vibration,
%    114(2), pp. 239-251, 1987.
% 3. R.A. Scott, "The propagation of sound between walls of porous materials", 
%    Proceedings of the Physical Society 58, pp. 358-368, 1946.
% 4. F.P. Mechel, "Taschenbuch der Technischen Akustik", Springer, Berlin, 1994.
% 5. J.F. Allard and N. Atalla, Propagation of Sound in Porous Media: 
%    Modelling Sound Absorbing Materials, 2nd edn., Wiley, 2009.
% 6. O. Doutres and Y. Salissou and N. Atalla and R. Panneton, 
%    "Evaluation of the acoustic and non-acoustic properties of sound 
%    absorbing materials using a three-microphone impedance tube", 
%    Applied Acoustics, 71, pp. 506-509, 2010.
%
% Author: BSC
% Date: 6/1/2013
%
% Revision History
% Date      Change 
% 

clc             % Clear the command window screen
close all       % Close all figures
clear all       % Clear the workspace


%% Define the System Parameters
%  Air 
rho_0 = 1.21                % [kg/m^3] Density of air
c_0 = 344                   % [m/s] Speed of sound in air
mu_0 = 18.3E-06             % [Pa.s] Dynamic viscosity 
gamma_0 = 1.4               % Specific heat ratio
K_0 = rho_0*c_0^2           % [kg/m/s^2] Ambient bulk modulus
P_0 = K_0/gamma_0           % [Pa] Static pressure

% Duct geometry
Duct_height = 0.25          % [m] Total duct height
h = Duct_height/2           % [m] Half duct height
l = 0.1                     % [m] Liner thickness

% Locally reacting liner impedance: Optimal impedance from Beranek
IMPD_Resistance = 0.92		% [N.s/m^3] Resistance of the absorbent 
IMPD_Reactance = -0.77		% [N.s/m^3] Reactance of the absorbent 

% Define the frequency vector and dependent parameters for the analysis
f = [1:1000];				% [Hz] Frequency
w = 2*pi*f;                 % [rad/s] Angular Frequency
k = w/c_0;                  % [1/m] Wavenumber
lambda_1 = c_0./f;          % [m] Wavelength
eta = 2*h./lambda_1;        % Ratio of duct height to wavelength


% Fluid properties for the bulk reacting liner.  
% This uses the Johnson-Champoux-Allard Equivalent Fluid Model
% Material properties were taken from Allard and Atalla
MAT_RESIS  = 10800          % [MKS Rayls/m] Flow resistivity
MAT_PORO   = 0.98           % Material porosity
MAT_TORTU  = 1.04           % Material tortuosity
MAT_VISCL  = 129e-6         % [m] Viscous char. length
MAT_THERL  = 198e-6         % [m] Thermal char. length
Prt = 0.713                 % Prandtl number at 20C

% Surface liner properties
rho_t = 0.0;                  % [kg/m^2] Surface density of facing 



%% Load Ansys APDL results
% Load the Complex Pressure and SPL for the upstream and downstream mics
% There are four cases:
% 1 - Rigid walled duct
% 2 - Locally reacting liner
% 3 - TBD - Bulk and local limp liner
% 4 - Bulk reacting liner


% For each load case
for count = 1:4
    % Source Mic
    % Load the Ansys file
    eval(['load ../ansys/apdl/lined_duct_',int2str(count),'a.txt'])
    % Frequency vector
    eval(['f_ansys',int2str(count),'a = lined_duct_', ...
        int2str(count),'a(:,1);'])
    % Complex Pressure
    eval(['P_ansys',int2str(count),'a = lined_duct_', ...
        int2str(count),'a(:,2:3)*[1;j];']) 
    % SPL
    eval(['SPL_ansys',int2str(count),'a = 20*log10(abs(P_ansys', ...
        int2str(count),'a))+91;'])
    % Sound power from the acoustic source
    eval(['Lw_',int2str(count),'a = 10*log10(1./(2*pi*f_ansys', ...
        int2str(count),'a)/rho_0.*abs( -real(P_ansys',int2str(count),'a) ))+120-3;'])
    
    
    % Termination Mic
    % Load the Ansys file
    eval(['load ../ansys/apdl/lined_duct_',int2str(count),'c.txt'])
    % Frequency vector
    eval(['f_ansys',int2str(count),' = lined_duct_', ...
        int2str(count),'c(:,1);'])
    % Complex Pressure
    eval(['P_ansys',int2str(count),' = lined_duct_', ...
        int2str(count),'c(:,2:3)*[1;j];']) 
    % SPL
    eval(['SPL_ansys',int2str(count),' = 20*log10(abs(P_ansys', ...
        int2str(count),'))+91;'])
    % IL across silencer
    % Difference in SPL between with and without element
    eval(['D_ansys_',int2str(count),' = SPL_ansys1 - SPL_ansys', ...
        int2str(count),';'])  
    % TL across silencer
    % Difference in incident to transmitted power
    eval(['TL_ansys_',int2str(count),' = D_ansys_',int2str(count), ...
        ' - (Lw_1a - Lw_',int2str(count),'a);'])
 
end




%% Locally Reacting Liner
% Model the attenuation provided by a locally reacting liner with
% an optimal frequency dependent impedance From Beranek, page 503
% 
% $$ Z_b = (R + jX) \rho_0 c_0 \eta $$
%
% The solution to transcendental equation is given by the complex
% propagation constant
%
% $$ \gamma_{x}=jk_{x}=jk_{0}\,\sqrt{1-\left(\frac{2}{k_{0}h}\right)^{2}\left[1+\frac{1}{1+\frac{4Z_{b}}{2k_{0}h\rho_{0}c_{0}}}\pm\sqrt{1+\frac{1}{\left(1+\frac{4Z_{b}}{2k_{0}h\rho_{0}c_{0}}\right)^{2}}}\right]}\,\,. $$
%
% And the attenuation (dB/m) per unit length of the duct is given by
%
% $$ \frac{D_h}{h} = 8.69 \Re \left(\gamma_{x}\right) $$
%

% Determine the optimal impedance
Z = (IMPD_Resistance + j*IMPD_Reactance)*rho_0*c_0*eta;
% Frequency independent impedance
f_fixed = 250               % [Hz] Frequency where attenuation will peak
Z = (IMPD_Resistance + j*IMPD_Reactance)*rho_0*c_0*(2.*h/(c_0./f_fixed))


% Complex propagation constant
% From Beranek, Eq (15.14)
Term1 = (2./k./h).^2;
Term2a = 4*Z./(j*k*h*rho_0*c_0);
Term2 = 1./(1 + Term2a );
Term3 = sqrt(1 + Term2.^2);
gamma_1 = j.*k.*sqrt( 1 - Term1.*(1 + Term2 - Term3)  );
gamma_2 = j.*k.*sqrt( 1 - Term1.*(1 + Term2 + Term3)  );
gamma_net = [gamma_1;gamma_2];

disp('Attenuation dB per unit length')
D = min(real(gamma_net)*8.69);

% Alternatively, the quadratic expression in Mechel
beta_1 = j*k*h*rho_0*c_0./Z;
Term1 = sqrt( (6203-j*857) + (2887.3 - j*372)*beta_1  + (867.4 - j*130)*beta_1.^2  );
E1 = ((78.94 - j*5.43) + beta_1*(34.47 - j*2.20) + Term1)./( 16.1 -j*1.11 + beta_1);
E2 = ((78.94 - j*5.43) + beta_1*(34.47 - j*2.20) - Term1)./( 16.1 -j*1.11 + beta_1);
gamma_1b = sqrt(E1 - (k*h).^2 )/h;
gamma_2b = sqrt(E2 - (k*h).^2 )/h;
gamma_netb = [gamma_1b;gamma_2b];
Db = min(real(gamma_netb)*8.69);


% From Beranek, page 503
disp('Approx max. attenuation dB per unit length')
Dmax = 19/h
% From Mechel
disp('Max. attenuation dB per unit length')
max(8.69*real( sqrt(3.42 -(k*h).^2 + j*5.24) )/h)

% Plot the results
figure('Name','Attenution from locally reacting liner optimised for 250Hz')
semilogx(f,D)
xlabel('Frequency (Hz)')
ylabel('Attenuation dB/m')
grid





%% Plot Locally Reacting results
% Create a new figure
figure('Name','Comparison of theory and Ansys for a locally reacting liner') 
semilogx(f,D,'k','LineWidth',2)     % Plot theoretical result
hold                                % Hold figure to overlay results
semilogx(f_ansys1,D_ansys_2,'k*','LineWidth',2)     % Plot Ansys IL
semilogx(f_ansys1,TL_ansys_2,'bo','LineWidth',2)    % Plot Ansys TL
set(gca, 'FontSize', 16)            % Increase font for printing
grid                                % Add a grid
xlabel('Frequency (Hz)')        
ylabel('Attenuation dB/m')
axis([10 1000 0 140])               % Fix axes
legend('MATLAB - Local','ANSYS - IL', 'ANSYS - TL','Location','NorthWest')
print -depsc 'lined_duct_locally_reacting.eps'







%% Johnson-Champoux-Allard Equivalent Fluid Model of a Porous Media
% This cell develops an equivalent fluid model which is necessary
% to model the bulk reacting liner.
%
% The effective density of the absorbent is given by
% J.F. Allard and N. Atalla, Propagation of Sound in Porous Media: 
% Modelling Sound Absorbing Materials, 2nd edn(Wiley, 2009).
%
% $$ \rho_{eff}=\frac{\sigma\phi}{j\omega}\left[1+\frac{j4\alpha_{\infty}^{2}\mu_{0}\omega\rho_{0}}{\phi^{2}\Lambda^{2}\sigma^{2}}\right]^{\frac{1}{2}}+\rho_{0}\alpha_{\infty} $$ 
%
% The effective bulk modulus of the absorbent is given by
% 
% $$ K_{eff}(\omega)=\frac{P_{0}\gamma}{\gamma-(\gamma-1)\left[\frac{8\mu_{0}}{j\omega\rho_{0}P_{rt}\Lambda'^{2}}\left(1+\frac{\Lambda'^{2}}{16}\frac{j\omega\rho_{0}P_{rt}}{\mu_{0}}\right)^{\frac{1}{2}}+1\right]^{-1}} $$ 
%

% Effective density of the absorbent
rho_eff = MAT_RESIS*MAT_PORO./(j*w).* ...
    ( 1 +  (j*4*MAT_TORTU^2*mu_0*w*rho_0)./ ...
    (MAT_PORO*MAT_VISCL*MAT_RESIS).^2 ).^(1/2) ...
    + rho_0*MAT_TORTU;

% Effective bulk modulus of the absorbent
K_eff = P_0*gamma_0./ ...
    (gamma_0 -(gamma_0 -1).*(8*mu_0./(j.*w*rho_0*Prt*MAT_THERL^2).* ...
    (1 + MAT_THERL^2*j*w*rho_0*Prt/(16*mu_0) ).^(1/2) + 1).^(-1) );

% Effective speed of sound in the absorbent
c_eff = sqrt(K_eff./rho_eff);

% Specific impedance of the absorbent
Zc = sqrt(K_eff.*rho_eff);

% Now using the above calculated dependent parameters, recalculate the 
% independent parameters to ensure the code is correct.
% The expressions below have come from Doutres at al.

disp('Check flow resistivity')
temp = - 1/MAT_PORO.*imag(w.*rho_eff);
MAT_RESIS_2 = temp(1);
[MAT_RESIS, MAT_RESIS_2]

disp('Check tortuosity')
temp = 1./rho_0.*(real(rho_eff) -  ...
    sqrt( imag(rho_eff).^2 - (MAT_RESIS*MAT_PORO./w).^2 ) ) ;
MAT_TORTU_2 = temp(1);
[MAT_TORTU, MAT_TORTU_2]

disp('Check Viscous char. length')
temp = MAT_TORTU.*sqrt( 2*rho_0*mu_0./(w.*imag(rho_eff).* ...
    (rho_0*MAT_TORTU - real(rho_eff)) )  );
MAT_VISCL_2 = temp(1);
[MAT_VISCL, MAT_VISCL_2] 

disp('Check Thermal char. length')
delta_t = sqrt(2*mu_0./(rho_0.*w*Prt));
temp = delta_t*sqrt(2).*( -imag( ( (1-K_eff/K_0)./ ...
    (1-gamma_0*K_eff/K_0) ).^2 )  ).^(-1/2);
MAT_THERL_2 = temp(1);
[MAT_THERL, MAT_THERL_2]




%% Bulk Reacting: Scott's method for an isotropic liner
% This cell calculates the solution to the transcendetal equation for a
% bulk reacting liner.
%
% For a limp liner with surface density $\rho_{m}$ and perforated sheet 
% with hole diameter $D_{s}$ the transcendental equation becomes
%
% $$ \sqrt{\left(k_{0}h\right)^{2}+\left(\gamma_{x}h\right)^{2}}\tan\sqrt{\left(k_{0}h\right)^{2}+\left(\gamma_{x}h\right)^{2}}= $$
%
% $$\frac{-j\rho_{0}c_{0}k_{0}}{Z_{l}\gamma_{l}}\sqrt{\left(\gamma_{x}h\right)^{2}-\left(\gamma_{l}h\right)^{2}}\tan\left(\frac{l}{h}\sqrt{\left(\gamma_{x}h\right)^{2}-\left(\gamma_{l}h\right)^{2}}\right) $$
%
% $$\times\left(1-\frac{\rho_{t}}{\rho_{0}h}\frac{j\rho_{0}c_{0}k_{0}}{Z_{l}\gamma_{l}}\sqrt{\left(\gamma_{x}h\right)^{2}-\left(\gamma_{l}h\right)^{2}}\tan\left(\frac{l}{h}\sqrt{\left(\gamma_{x}h\right)^{2}-\left(\gamma_{l}h\right)^{2}}\right)\right)^{-1} $$
%


Gamma_l = [];           % Initialise the complex propagation constant matrix
Gamma_local = [];       % Initialise the complex propagation constant matrix
D_l = [];               % Initialise the attenutation dB/m matrix
D_local = [];           % Initialise the attenutation dB/m matrix
x0 = [.1,.1]            % Initial estimate of the complex propagation constant

% Now of each frequency of interest determine the attenuation
for count=1:length(f)
    
    freq = f(count);         % [Hz] Frequency
    omega = 2*pi*freq;       % [rad/s] Angular frequency
    k_0 = omega/c_0;         % [1/m] Wavenumber

    gamma_l = j*omega/c_eff(freq);  % Bulk propagation constant of the absorbent
    Zl = Zc(freq);                  % Characteristic impendance of the absorbent

    % Solve for the complex propagation constant in the bulk liner
    % Define input parameters necessary to solve the problem
    input_params = [k_0, l, h, rho_0, c_0, Zl, gamma_l,rho_t];
    % Define the structure of the call to the solver
    % The function is defined in a separate file called Scott
    fx = @(x)scott(x,input_params);
    % Call the solver.  It will return two values; the real and imag terms
    [x,fval] = fsolve(fx,x0,optimset('Display','off') );
    gamma_l = complex(x(1),x(2));           % Returned complex propagation constant
    x0 = [real(gamma_l),imag(gamma_l)];     % Let the initial guess be current value 

    Gamma_l(count) = gamma_l;               % Store current result in complex propagation constant matrix
    D_l(count) = 8.69*real(gamma_l);        % Store current result in attenuation matrix
    
    % Compare against a locally reacting equivalent
    % Complex propagation constant
    % From Beranek, Eq (15.14)
    Term1 = (2./k_0./h).^2;
    Term2a = 4*Zl./(j*k_0*h*rho_0*c_0);
    Term2 = 1./(1 + Term2a );
    Term3 = sqrt(1 + Term2.^2);
    gamma_1 = j.*k_0.*sqrt( 1 - Term1.*(1 + Term2 - Term3)  );
    gamma_2 = j.*k_0.*sqrt( 1 - Term1.*(1 + Term2 + Term3)  );
    gamma_net = [gamma_1;gamma_2];
    Gamma_local(count) = gamma_1;               % Complex propagation constant matrix
    D_local(count) = 8.69*min(real(gamma_net)); % Attenuation matrix
end



%% Plot Bulk Reacting results
% Open a new figure
figure('Name','Comparison of theory and Ansys for a bulk reacting liner') 
semilogx(f,D_l,'k','LineWidth',2)       % Bulk reacting theory
hold                                    % Hold the figure
semilogx(f,D_local,'k--','LineWidth',2) % Locally reacting
semilogx(f_ansys1,D_ansys_4,'k*','LineWidth',2)     % IL from Ansys
semilogx(f_ansys1,TL_ansys_4,'bo','LineWidth',2)    % TL from Ansys
set(gca, 'FontSize', 16)                % Increase font for printing
grid                                    % Add a grid
xlabel('Frequency (Hz)')
ylabel('Attenuation dB/m')
legend('MATLAB - Bulk','MATLAB - Local','ANSYS - IL','ANSYS - TL','Location','NorthWest')
axis([100 1000 0 30])
print -depsc 'lined_duct_bulk_reacting.eps'





%% Load Ansys WorkBench results
% Load the Complex Pressure and SPL for the upstream and downstream mics
% Calculations assume a mass source
% There are three cases:
% 1 - Rigid walled duct
% 2 - Locally reacting liner
% 3 - Bulk reacting liner

% Load the source mic data
lined_duct_1a = textread('../ansys/workbench/Lined_Duct_Rigid_Source_Pres.txt',...
    '','headerlines',17,'delimiter',' ');
f_ansys1a = lined_duct_1a(:,1);                         % Frequency vector
P_ansys1a = lined_duct_1a(:,2) + j*lined_duct_1a(:,3);  % Complex Pressure
SPL_ansys1a = 20*log10(abs(P_ansys1a))+91;              % SPL vector
% Sound power from the acoustic source
Lw_1a = 10*log10(1./rho_0.*abs( -real(P_ansys1a) ))+120-3;

lined_duct_2a = textread('../ansys/workbench/Lined_Duct_Local_Source_Pres.txt', ...
    '','headerlines',17,'delimiter',' ');
f_ansys2a = lined_duct_2a(:,1);                         % Frequency vector
P_ansys2a = lined_duct_2a(:,2) + j*lined_duct_2a(:,3);  % Complex Pressure
SPL_ansys2a = 20*log10(abs(P_ansys2a))+91;              % SPL vector
% Sound power from the acoustic source
Lw_2a = 10*log10(1./rho_0.*abs( -real(P_ansys2a) ))+120-3;

lined_duct_3a = textread('../ansys/workbench/Lined_Duct_Bulk_Source_Pres.txt','', ...
    'headerlines',17,'delimiter',' ');
f_ansys3a = lined_duct_3a(:,1);                         % Frequency vector
P_ansys3a = lined_duct_3a(:,2) + j*lined_duct_3a(:,3);  % Complex Pressure
SPL_ansys3a = 20*log10(abs(P_ansys3a))+91;              % SPL vector
% Sound power from the acoustic source
Lw_3a = 10*log10(1./rho_0.*abs( -real(P_ansys3a) ))+120-3;


% Termination mic
lined_duct_1c = textread('../ansys/workbench/Lined_Duct_Rigid_Termination_SPL.txt', ...
    '','headerlines',17,'delimiter',' ');
f_ansys1c = lined_duct_1c(:,1);                          % Frequency vector
SPL_ansys1c = lined_duct_1c(:,2);                        % SPL vector

lined_duct_2c = textread('../ansys/workbench/Lined_Duct_Local_Termination_SPL.txt', ...
    '','headerlines',17,'delimiter',' ');
f_ansys2c = lined_duct_2c(:,1);                          % Frequency vector
SPL_ansys2c = lined_duct_2c(:,2);                        % SPL vector
D_ansys_2c = SPL_ansys1c - SPL_ansys2c;                    % IL across silencer
TL_ansys_2c = SPL_ansys1c - SPL_ansys2c - (Lw_1a - Lw_2a); % TL across silencer

lined_duct_3c = textread('../ansys/workbench/Lined_Duct_Bulk_Termination_SPL.txt', ...
    '','headerlines',17,'delimiter',' ');
f_ansys3c = lined_duct_3c(:,1);                          % Frequency vector
SPL_ansys3c = lined_duct_3c(:,2);                        % SPL vector
D_ansys_3c = SPL_ansys1c - SPL_ansys3c;                    % IL across silencer
TL_ansys_3c = SPL_ansys1c - SPL_ansys3c - (Lw_1a - Lw_3a); % TL across silencer



%% Plot the Ansys WorkBench results
% Create a new figure
figure('Name','Comparison of theory and Workbench for a locally reacting liner') 
semilogx(f,D,'k','LineWidth',2)         % Plot theoretical result
hold                                    % Hold figure to overlay results
semilogx(f_ansys1,D_ansys_2c,'k*','LineWidth',2)     % Plot Ansys IL
semilogx(f_ansys1,TL_ansys_2c,'bo','LineWidth',2)    % Plot Ansys TL
set(gca, 'FontSize', 16)                % Increase font for printing
grid                                    % Add a grid
xlabel('Frequency (Hz)')        
ylabel('Attenuation dB/m')
axis([10 1000 0 140])                   % Fix axes
legend('MATLAB - Local','ANSYS - IL', 'ANSYS - TL','Location','NorthWest')
print -depsc 'lined_duct_locally_reacting_WB.eps'


% Plot Bulk Reacting Results
% Open a new figure
figure('Name','Comparison of theory and Workbench for a bulk reacting liner') 
semilogx(f,D_l,'k','LineWidth',2)       % Bulk reacting theory
hold                                    % Hold the figure
semilogx(f_ansys1,D_ansys_3c,'k*','LineWidth',2)     % IL from Ansys
semilogx(f_ansys1,TL_ansys_3c,'bo','LineWidth',2)    % TL from Ansys
set(gca, 'FontSize', 16)                % Increase font for printing
grid                                    % Add a grid
xlabel('Frequency (Hz)')
ylabel('Attenuation dB/m')
legend('MATLAB - Bulk','ANSYS - IL','ANSYS - TL','Location','NorthWest')
axis([100 1000 0 30])
print -depsc 'lined_duct_bulk_reacting_wb.eps'