function fx = scott(gamma,input_params)
% This function is used to solve the complex propagation constant in a 
% lined duct.

% It is based on the paper by Scott, R.A. (1946) "The propagation of
% sound between walls of porous materials", Proceedings of the Physical
% Society, 58, 358-368.
%
% The actual formula was taken directly from Wassilieff, C. (1987),
% "Experimental verification of duct attenuation models with bulk reacting
% liners", Journal of Sound and Vibration, 114(2), 239-251.
%
% Note that the absorbent is assumed to be isotropic.
% 
% Author: BSC
% Date: 6/1/2013
%
% Revision History
% Date          Change 
% 20/1/2013     Added surface density of liner
%

%% Check input parameters
if length(gamma)==2
    gamma = complex(gamma(1),gamma(2));
else
    error('Gamma must be real and imaginary')
end

if length(input_params)==8
    % Extract the input parameters
    k_0 = input_params(1);      % Wavenumber in freespace
    l = input_params(2);        % Liner thickness
    h = input_params(3);        % Half duct thickness
    rho_0 = input_params(4);    % Density of air
    c_0 = input_params(5);      % Speed of sound in air
    Zl = input_params(6);       % Liner characteristic impedance
    gamma_l = input_params(7);  % Bulk propagation constant in liner
    rho_t = input_params(8);    % Surface density of facing 
else
    error('The input parameter vector must be of length 8')
end


%% Define the function
% From Eqn (1), Wassilieff

% Left hand terms
Term_a1 = sqrt( (k_0*h)^2 + (gamma*h)^2 );
Term_a2 = tan( Term_a1 );
% Right hand terms
Term_b1 = -j*rho_0*c_0*k_0/(Zl*gamma_l);
Term_b2 = sqrt( (gamma*h)^2  - (gamma_l*h)^2  );
Term_b3 = tan( (l/h)*Term_b2  );
Term_b4 = (1 +  rho_t/(rho_0*h)*Term_b1*Term_b2*Term_b3);

fx = -Term_a1*Term_a2 + Term_b1*Term_b2*Term_b3/Term_b4;
fx = [real(fx);imag(fx)];           %Separate as real and imaginary
