function [c]=loadcav(filename)
% load cavity information from Ansys in to Matlab
%
% currently requires information to be written out from Ansys in specific
% format (see documentation). Loading subfunctions are abstracted so that
% later they can be replaced with functions that directly load ansys .rst
% and .mode files.
%
% returns information in a structure
% 
% c.n  -> nodal information
% c.e  -> elemental information
% c.f  -> frequency information
% c.p  -> mode shapes
% c.m  -> modal mass
% c.pl -> plots

[n,e]=loadcavgeom(filename);

% map node index to number
n.idx=zeros(max(n.num),1);
for ii=1:length(n.num);n.idx(n.num(ii))=ii;end

% remove nodes not used in elements
elem_refnodes=union(e.i,e.j);
elem_refnodes=union(elem_refnodes,e.k);
elem_refnodes=union(elem_refnodes,e.l);
elem_refnodes=union(elem_refnodes,e.m);
elem_refnodes=union(elem_refnodes,e.n);
elem_refnodes=union(elem_refnodes,e.o);
elem_refnodes=union(elem_refnodes,e.p);

n.num=n.num(n.idx(elem_refnodes));
n.x=n.x(n.idx(elem_refnodes));
n.y=n.y(n.idx(elem_refnodes));
n.z=n.z(n.idx(elem_refnodes));

% remap node index to number
n.idx=zeros(max(n.num),1);
for ii=1:length(n.num);n.idx(n.num(ii))=ii;end

% cal elem areas and normals
% element points
pi=[n.x(n.idx(e.i)),n.y(n.idx(e.i)),n.z(n.idx(e.i))];
pj=[n.x(n.idx(e.j)),n.y(n.idx(e.j)),n.z(n.idx(e.j))];
pk=[n.x(n.idx(e.k)),n.y(n.idx(e.k)),n.z(n.idx(e.k))];
pl=[n.x(n.idx(e.l)),n.y(n.idx(e.l)),n.z(n.idx(e.l))];
pm=[n.x(n.idx(e.m)),n.y(n.idx(e.m)),n.z(n.idx(e.m))];
pn=[n.x(n.idx(e.n)),n.y(n.idx(e.n)),n.z(n.idx(e.n))];
po=[n.x(n.idx(e.o)),n.y(n.idx(e.o)),n.z(n.idx(e.o))];
pp=[n.x(n.idx(e.p)),n.y(n.idx(e.p)),n.z(n.idx(e.p))];

% element vectors
rij=0.5*(pj-pi);
rjk=0.5*(pk-pj);
rkl=0.5*(pl-pk);
rli=0.5*(pi-pl);
rmn=0.5*(pn-pm);
rno=0.5*(po-pn);
rop=0.5*(pp-po);
rpm=0.5*(pm-pp);
rim=0.5*(pm-pi);
rjn=0.5*(pn-pj);
rko=0.5*(po-pk);
rlp=0.5*(pp-pl);

%element areas
ai=dot(rij,cross(-rli,rim),2);
aj=dot(rjk,cross(-rij,rjn),2);
ak=dot(rkl,cross(-rjk,rko),2);
al=dot(rli,cross(-rkl,rlp),2);
am=dot(-rpm,cross(rmn,-rim),2);
an=dot(-rmn,cross(rno,-rjn),2);
ao=dot(-rno,cross(rop,-rko),2);
ap=dot(-rop,cross(rpm,-rlp),2);

% nodal area and norm
% must loop through elements, otherwise repeated nodes are not indexed
% properly
n.vol=zeros(size(n.num));

for ii=1:length(e.num)
    elmidx=[n.idx(e.i(ii)),n.idx(e.j(ii)),n.idx(e.k(ii)),n.idx(e.l(ii)),...
            n.idx(e.m(ii)),n.idx(e.n(ii)),n.idx(e.o(ii)),n.idx(e.p(ii))];
    n.vol(elmidx)=n.vol(elmidx)+[ai(ii),aj(ii),ak(ii),al(ii),am(ii),an(ii),ao(ii),ap(ii)]';
end


mode=loadcavmode(filename);
mode.num=mode.num(1:length(mode.num)/length(mode.freq));

mode.p=reshape(mode.p,[],length(mode.freq));

% map mode node index to number
mode.idx=zeros(max(mode.num),1);
for ii=1:length(mode.num);mode.idx(mode.num(ii))=ii;end

mode.num=mode.num(mode.idx(n.num));
mode.p=mode.p(mode.idx(n.num),:);

% remap mode node index to number
mode.idx=zeros(max(mode.num),1);
for ii=1:length(mode.num);mode.idx(mode.num(ii))=ii;end

% mode.mass = 1./(max(abs(mode.psi))).^2;
% mode.psi = mode.psi.*(ones(length(mode.num),1)*sqrt(mode.mass)); % Normalise the mode
mode.volume = (1./max(abs(mode.p))).^2*n.c.^2;
mode.p = mode.p./(ones(length(mode.num),1)*max(mode.p)); % Normalise the mode

% return structure
c.n=rmfield(n,{'c','dens'});
c.e=e;
c.f=mode.freq';  % frequency information
c.p=mode.p;   % mode shapes
c.mv=mode.volume;  % modal mass
c.c=n.c;
c.dens=n.dens;
c.id='cav';

function [n,e]=loadcavgeom(filename);
% load geometry information
% currently assumes that elements used are 8 node bricks.
% Must also read in properties of air at which modes were calculated. This
% must be written out in ansys,speed of sound then density

% load node and element data saved from ansys
[n.num,n.x,n.y,n.z] =  textread([filename,'.nde'],'%n%n%n%n');
[e.i,e.j,e.k,e.l,e.m,e.n,e.o,e.p,e.mat,e.type,e.real,e.num] ...
    =  textread([filename,'.elm'],'%n%n%n%n%n%n%n%n%n%n%n%*n%*n%n');
props=textread([filename,'.prp'],'%s',-1);

% From the *.cav.prp file that was imported,
% get the speed of sound which is termed SONC in the material properties.
% Note that as versions of ANSYS change, the number props{21} might need to
% be altered.
n.c=str2num(props{21});
% From the *.cav.prp file that was imported,
% get the density of the fluid which is termed DENS in the 
% material properties.
% Note that as versions of ANSYS change, the number props{15} might need to
% be altered.
n.dens=str2num(props{15});

function [m]=loadcavmode(filename)
% load modal information
% must be mass normalised results from Ansys (not unity normalised)

m.freq=textread([filename,'.frq'],'%*n%n%*n%*n%*n','headerlines',5);
[m.num,m.p]=textread([filename,'.mde'],'%n%n');
