function [s]=loadstr(filename)
% load structural information from Ansys in to Matlab
%
% currently requires information to be written out from Ansys in specific
% format (see documentation). Loading subfunctions are abstracted so that
% later they can be replaced with functions that directly load ansys .rst
% and .mode files.
%
% returns information in a structure
% 
% s.n  -> nodal information
% s.e  -> elemental information
% s.f  -> frequency information
% s.p  -> mode shapes
% s.m  -> modal mass
% s.pl -> plots
% s.id -> id string ('str')

[n,e]=loadstrgeom(filename);

% map node index to number
n.idx=zeros(max(n.num),1);
for ii=1:length(n.num);n.idx(n.num(ii))=ii;end

% remove nodes not used in elements
elem_refnodes=union(e.i,e.j);
elem_refnodes=union(elem_refnodes,e.k);
elem_refnodes=union(elem_refnodes,e.l);

n.num=n.num(n.idx(elem_refnodes));
n.x=n.x(n.idx(elem_refnodes));
n.y=n.y(n.idx(elem_refnodes));
n.z=n.z(n.idx(elem_refnodes));

% remap node index to number
n.idx=zeros(max(n.num),1);
for ii=1:length(n.num);n.idx(n.num(ii))=ii;end

% cal elem areas and normals
% element points
e_pi=[n.x(n.idx(e.i)),n.y(n.idx(e.i)),n.z(n.idx(e.i))];
e_pj=[n.x(n.idx(e.j)),n.y(n.idx(e.j)),n.z(n.idx(e.j))];
e_pk=[n.x(n.idx(e.k)),n.y(n.idx(e.k)),n.z(n.idx(e.k))];
e_pl=[n.x(n.idx(e.l)),n.y(n.idx(e.l)),n.z(n.idx(e.l))];

% element vectors
e_r1=0.5*(e_pj-e_pi);
e_r2=0.5*(e_pk-e_pj);
e_r3=0.5*(e_pl-e_pk);
e_r4=0.5*(e_pi-e_pl);

%element normals
e_ni=cross(e_r1,-e_r4);
e_nj=cross(e_r2,-e_r1);
e_nk=cross(e_r3,-e_r2);
e_nl=cross(e_r4,-e_r3);

%element areas
e_ai=sqrt(sum(e_ni.^2,2));
e_aj=sqrt(sum(e_nj.^2,2));
e_ak=sqrt(sum(e_nk.^2,2));
e_al=sqrt(sum(e_nl.^2,2));

% nodal area and norm
% must loop through elements, otherwise repeated nodes are not indexed
% properly
n.area=zeros(size(n.num));
n.norm=zeros(length(n.num),3);

for ii=1:length(e.num)
    elmidx=[n.idx(e.i(ii)),n.idx(e.j(ii)),n.idx(e.k(ii)),n.idx(e.l(ii))];
    n.area(elmidx)=n.area(elmidx)+[e_ai(ii),e_aj(ii),e_ak(ii),e_al(ii)]';
    n.norm(elmidx,:)=n.norm(elmidx,:)+[e_ni(ii,:);e_nj(ii,:);e_nk(ii,:);e_nl(ii,:)];
end

n.norm=n.norm./(sqrt(sum(n.norm.^2,2))*[1 1 1]); % unity normalised

e.idx=zeros(max(e.num),1);
for ii=1:length(e.num);e.idx(e.num(ii))=ii;end

mode=loadstrmode(filename);
mode.num=mode.num(1:length(mode.num)/length(mode.freq));

mode.x=reshape(mode.x,[],length(mode.freq));
mode.y=reshape(mode.y,[],length(mode.freq));
mode.z=reshape(mode.z,[],length(mode.freq));

% map mode node index to number
mode.idx=zeros(max(mode.num),1);
for ii=1:length(mode.num);mode.idx(mode.num(ii))=ii;end

mode.num=mode.num(mode.idx(n.num));
mode.x=mode.x(mode.idx(n.num),:);
mode.y=mode.y(mode.idx(n.num),:);
mode.z=mode.z(mode.idx(n.num),:);

% remap mode node index to number
mode.idx=zeros(max(mode.num),1);
for ii=1:length(mode.num);mode.idx(mode.num(ii))=ii;end

mode.psi=zeros(length(mode.num),length(mode.freq));
for ii = 1:length(mode.freq);
    mode.psi(:,ii)=dot([mode.x(:,ii),mode.y(:,ii),mode.z(:,ii)],n.norm,2);
end
mode.mass = 1./(max(abs(mode.psi))).^2;
mode.psi =  mode.psi.*(ones(length(mode.num),1)  ...
            *sqrt(mode.mass)); % Normalise the mode

% return structure
s.n=n;
s.e=e;
s.f=mode.freq';  % frequency information
s.p=mode.psi;   % mode shapes
s.mm=mode.mass;  % modal mass
s.id='str';

function [n,e]=loadstrgeom(filename);
% load geometry information
% currently assumes that elements used are 4 node shells. This is not
% necessarily a big restriction. Composite 8 node elements (shell99) can be
% used as the last 4 nodes are not used. Solids could conceivably be used
% by convering them with dummy shell elements (mesh200). Needs to be
% rethought at some stage...

% load node and element data saved from ansys
[n.num,n.x,n.y,n.z] =  textread([filename,'.nde'],'%n%n%n%n');
[e.i,e.j,e.k,e.l,e.mat,e.type,e.real,e.num] ...
    =  textread([filename,'.elm'],'%n%n%n%n%*n%*n%*n%*n%n%n%n%*n%*n%n');

function [m]=loadstrmode(filename)
% load modal information
% must be mass normalised results from Ansys (not unity normalised)

m.freq=textread([filename,'.frq'],'%*n%n%*n%*n%*n','headerlines',5);
[m.num,m.x,m.y,m.z]=textread([filename,'.mde'],'%n%n%n%n%*n%*n%*n');
