%% Calculate Structural Kinetic Energy From Full FSI Ansys Analysis
% This script can be used to calculate the structural kinetic energy from
% conducting a full harmonic analysis, where the displacement 
% at every node and every frequency has been exported.

%% Check existance of m.s.n.area
% Before calculting the results, make sure that the modal data has already
% been processed, so that the area associated with each structural node 
% has been calculated and available in the matlab structure m.s.n.area
if ~isfield(m.s.n,'area')
    disp('Use the loadmodel.m script first to calculate the nodal areas');
    error('Use the Ansys modal coupling scripts first.');
end;

%% Check that the density and thickness have been defined
% Before calculating the results check that the density and thickness of
% the plate have been defined. 
% These data are not determined from the Ansys script |extract_modes.mac|
if ~exist('rho_structure','var')
    error('Define the density of the structure rho_structure and run again.');
end;

if ~exist('thick_structure','var')
    error('Define the density of the structure thick_structure and run again.');
end;

%% Check that the variable ansys_freq_full exists
% This is a vector that contains the analysis frequencies from the FSI full
% harmonic analysis.
%
% We can also check that the size of ansys_freq is the same length as the
% displacement data, which will be done later
if ~exist('ansys_freq_full','var')
    error('Define the ansys_freq_full variable.');
end;


%% Load the exported Ansys results
% Load the real displacement at every node and every frequency.
% This could take a while
d_real=load('real_d.txt');
% Convert the data into an array
d_real=readansysarray(d_real);

%%%
% Load the imaginary displacement at every node and every frequency.
% This could take a while
d_imag=load('imag_d.txt');
% Convert the data into an array
d_imag=readansysarray(d_imag);

%%% 
% Put the real and imaginary displacement results into a single variable
% which has dimensions [nodes x frequencies]
displacements=d_real+1i*d_imag;
clear d_real
clear d_imag

%% Check the size of frequency vector matches displacement data
if length(ansys_freq_full)~=size(displacements,2)
    error('The length of the ansys_freq vector does not match size of ansys data');
end;


%% Calculate the Structural Energy Energy
% The structural kinetic energy can be calculated as
%
% $$
% E_{k}({\omega}) = \frac{1}{2} \, 
%                \sum_{n=1}^{N_{a}} |j \omega u_{n}(\omega)|^{2} M_{n}
% $$
%
% $$
% E_{k}({\omega}) = \frac{\omega^2 \times \rho_{\textrm{plate}} t}{2} \, 
%                \sum_{n=1}^{N_{a}} |u_{n}(\omega)|^{2} A_{n}
% $$
% 
% where
%
% * $\omega$ is the circular frequency
% * $N_a$ is the number of nodes
% * $u_{n}(\omega)$ is the structural displacement at the nth node 
% at circular frequency $\omega$ 
% * $M_n$ is the mass associated with the nth node
% * $A_n$ is the area associated with the nth node
% * $\rho_{\textrm{plate}}$ is the density of the plate
% * $t$ is the thickness of the plate
%
%
nodal_area=zeros(size(displacements,2),1);
for ii=1:length(m.s.n.num)
    nodal_area(m.s.n.num(ii))=m.s.n.area(ii);
end;

for ff=1:size(displacements,2)
    temp_ske(ff)=   (2*pi*ansys_freq_full(ff)).^2   ...
                    *sum(abs(displacements(:,ff)).^2 .* nodal_area);
end;

% Calculate the structural kinetic energy
ansys_ske_long_way=1/2*rho_structure*thick_structure*temp_ske;


 
