%% Vibro-Acoustic Response of A Cavity With A Flexible Plate
% Usage: theory_couple_plate_cavity
%
% Calculates the vibro-acoustic response of a simply supported 
% rectangular plate attached to the end wall of a rigid cavity. 
% A harmonic point force acts on the plate, causing it to vibrate and
% generate sound in the cavity. The displacement of the plate is measured
% at the location of the driving force and the sound pressure in the cavity
% is measured at a location that is coincident with a node of an equivalent
% finite element model.
%
% Carl Howard 12 April 2013


%% Dimensions of the cavity
cavity.len_x = 0.5;    % [m]
cavity.len_y = 0.3;    % [m]
cavity.len_z = 1.1;    % [m]
cavity.volume = cavity.len_x .* cavity.len_y .* cavity.len_z;   % [m^3]

%% Parameters of the fluid
fluid.density   = 1.21;     % [kg/m^3] density of the gas
fluid.speed     = 344;      % [m/s] speed of sound of the gas

%% Dimensions of the plate
plate.len_x = cavity.len_x;     % [m]
plate.len_y = cavity.len_y;     % [m]
plate.thick = 0.003;            % [m]

%% Material properties of Aluminium
plate.density	= 2700;             % [kg/m^3] density
plate.E         = 70.9e9;           % [Pa] Youngs modulus
plate.mu        = 0.3;              % [no units] Poissons Ratio


%% Define location of acoustic source
speaker.loc_x=0.1;
speaker.loc_y=0.1;
speaker.loc_z=0.1;
speaker.vol_velocity=0;


%% Define the location of the microphone
mic.loc_x=0.125;
mic.loc_y=0.150;
mic.loc_z=0.875;


%mic.loc_z=cavity.len_z-0.875;


% % It is possible to define a mesh for the microphone locations
% mic.loc_x=0;
% mic.loc_y=0;
% mic.loc_z=0;
% % Division of the rectangular plate and cavity
% div.x   =   20;
% div.y   =   20;
% div.z   =   20;
% 
% for kk=0:cavity.len_z./div.z:cavity.len_z,
% 	for jj=0:cavity.len_y./div.y:cavity.len_y,
% 		for ii=0:cavity.len_x./div.x:cavity.len_x,
%             mic.loc_x=[mic.loc_x;ii];
%             mic.loc_y=[mic.loc_y;jj];
%             mic.loc_z=[mic.loc_z;kk];
%         end; %for ii
%     end; %for jj
% end; % for kk
% 
% % remove the first entry
% mic.loc_x=mic.loc_x(2:size(mic.loc_x,1));
% mic.loc_y=mic.loc_y(2:size(mic.loc_y,1));
% mic.loc_z=mic.loc_z(2:size(mic.loc_z,1));


%% Define the load on the plate
% The example is for a point load on the plate
force.loc_x  = 0.1;
force.loc_y  = 0.075;
force.amp    = 1;        % amplitude of the force


%% Define location of accelerometer
% Make the accelerometer location at the same location where the load is
% applied
accel.loc_x = force.loc_x;
accel.loc_y = force.loc_y;


% create an array of accelerometer locations on the the plate
% if we just copy the entries from the acoustic space, then we know
% that the order of the location matrix for the structure and acoustics is
% identical.
% accel.loc_x = mic.loc_x(1:(div.x+1).*(div.y+1));
% accel.loc_y = mic.loc_y(1:(div.x+1).*(div.y+1));


%%%
% The mode shape for a plate is 
%
% $$ \sin \left( \frac{m_1 \pi x}{L_x} \right) 
%       \times \sin \left( \frac{m_2 \pi  y}{L_y} \right) $$
%


% convert the previous definitions into terms used in the paper

surfacearea = plate.len_x .* plate.len_y;
speedair  = fluid.speed;
densair  = fluid.density;

%% Define the number of structural and acoustic modes to use
nstrmodes   =   21;     % number of structural modes to use
ncavmodes   =   102;    % number of acoustic modes to use


%% Define the analysis frequency range
freq_vect = [1:400];                        % [Hz]
end_freq = freq_vect(length(freq_vect));


%% Calculate the resonance frequencies of the cavity
% Create a matrix containing all the mode index permutations.
% Start by defining a matrix that is 
% |n_max|^3 the number of modes to calculate the resonance frequencies
% which will be sorted and truncated to num_modes
n_max = 30;

fprintf('===Start calculation of the resonance frequencies of the cavity\n');


mode_index=[];
for ii=0:n_max
    for jj = 0:n_max
        for kk = 0:n_max
            mode_index = [ mode_index ; ii jj kk ];
        end;
    end;
end;

%% Calculate the resonance frequencies of the cavity
res_freq = sqrt(    (mode_index(:,1) / cavity.len_x).^2 +   ...
                    (mode_index(:,2) / cavity.len_y).^2 +   ...    
                    (mode_index(:,3) / cavity.len_z).^2 );

res_freq = res_freq .* fluid.speed ./ 2;	% [Hz]

% sort the resonance frequencies
temp=sortrows([res_freq mode_index],1);
res_freq = temp(:,1);
mode_index = temp(:,2:4);

% Use only first 1:ncavmodes
cavity.res_freq = res_freq(1:ncavmodes);
cavity.mode_index = mode_index(1:ncavmodes,:);
 

%% Calculate the modal volume
% see Beranek and Ver book P162
epsilon_modal_volume = zeros(size(cavity.mode_index));
index2change = find(cavity.mode_index(:,1)==0);
epsilon_modal_volume(index2change,1) = 1;
index2change = find(cavity.mode_index(:,1)>0);
epsilon_modal_volume(index2change,1) = 0.5;
index2change = find(cavity.mode_index(:,2)==0);
epsilon_modal_volume(index2change,2) = 1;
index2change = find(cavity.mode_index(:,2)>0);
epsilon_modal_volume(index2change,2) = 0.5;
index2change = find(cavity.mode_index(:,3)==0);
epsilon_modal_volume(index2change,3) = 1;
index2change = find(cavity.mode_index(:,3)>0);
epsilon_modal_volume(index2change,3) = 0.5;

cavity.modalvolume = cavity.volume         ...
            .* epsilon_modal_volume(:,1)    ...
            .* epsilon_modal_volume(:,2)    ...
            .* epsilon_modal_volume(:,3);


%% Evaluate the mode shapes at the microphone locations
cavity.phi_eval =  ...
    cos(cavity.mode_index(:,1)*pi*mic.loc_x.'/cavity.len_x) .*   ...
    cos(cavity.mode_index(:,2)*pi*mic.loc_y.'/cavity.len_y) .*   ...
    cos(cavity.mode_index(:,3)*pi*mic.loc_z.'/cavity.len_z);


%% Evaluate the mode shapes at the acoustic source location
speaker.phi_eval =  ...
    cos(cavity.mode_index(:,1)*pi*speaker.loc_x.'/cavity.len_x) .*   ...
    cos(cavity.mode_index(:,2)*pi*speaker.loc_y.'/cavity.len_y) .*   ...
    cos(cavity.mode_index(:,3)*pi*speaker.loc_z.'/cavity.len_z);


fprintf('===End of calculation of the resonance frequencies of the cavity \n');



%% Calculate the resonance frequencies of the plate
% Create a matrix containing all the mode index permutations
%
% |n_max|^3 the number of modes to calculate the resonance frequencies
% which will be sorted and truncated to num_modes
n_max = 30;

fprintf('===Start calculation of the resonance frequencies of the plate\n');

str_mode_index=[];
for ii=1:n_max
    for jj = 1:n_max
            str_mode_index = [ str_mode_index ; ii jj ];
    end;
end;

% Calculate the resonance frequencies of the plate
% This formula was taken from Leissa
% The formula in the Bokil paper is incorrect.
D = plate.E .* plate.thick.^3 ./ (12.*(1 - plate.mu.^2));
res_freq =  1./(2.*pi) .*  ...
            sqrt(D ./ plate.density ./ plate.thick ) .*   ...
            (   (str_mode_index(:,1).*pi ./ plate.len_x).^2 +   ...
                (str_mode_index(:,2).*pi ./ plate.len_y).^2  );
            
% sort the resonance frequencies
temp=sortrows([res_freq str_mode_index],1);
res_freq = temp(:,1);
str_mode_index = temp(:,2:3);

% use only first 1:nstrmodes
plate.res_freq = res_freq(1:nstrmodes);
plate.mode_index = str_mode_index(1:nstrmodes,:);



%% Calculate the modal mass of the plate
plate.modalmass =   plate.density          ...
                    .* plate.thick      ...
                    .* plate.len_x      ...
                    .* plate.len_y      ...
                    ./ 4 * ones(1,nstrmodes);

        
%% Calculate the force load vector in terms of modal participation
% see eq (11)
%
% The modal forcing function is calculated as
%
% $$ F_m = F_z \times \sin \left( \frac{m_1 \pi x_F}{L_x} \right) 
%       \times \sin \left( \frac{m_2 \pi  y_F}{L_y} \right) $$
%
% where
%
% * $F_z$ is the amplitude of the force.
% * $(x_F,y_F)$ is the coordinate on the plate where the force is applied
%
force_modal = - force.amp .*   ...
    sin(plate.mode_index(:,1).*pi.*force.loc_x ./ plate.len_x) .*   ...
    sin(plate.mode_index(:,2).*pi.*force.loc_y ./ plate.len_y);

%% Calculate speaker acoustic load vector in terms of modal participation
% The acoustic source is defined as a volume acceleration which is
% calculated as
%
% $$ \dot{Q} = j \omega Q $$
%
% Then the modal acoustic volume velocity excitation is
%
% $$ \dot{\rm{Q}}_m = \dot{Q} \phi $$
%
% For the moment just calculate the $Q_m$ and the volume acceleration
% $\dot{\rm{Q}}_m= j \omega \rm{Q}_m$ will be calculated 
% in the frequency loop.
Q_speaker =     speaker.vol_velocity        ...
                    * speaker.phi_eval;
      
%% Calculate the mode shapes at the accelerometer locations      
%
% $$ \psi(x_a,y_a) = 
%       \sin \left( \frac{m_1 \pi x_a}{L_x} \right) 
%       \times \sin \left( \frac{m_2 \pi  y_a}{L_y} \right) $$
%
% where
%
% * $(x_a,y_a)$ is the coordinate of the accelerometer on the plate.
%
plate.psi_eval =    ...
            sin(plate.mode_index(:,1)*pi*accel.loc_x.'/ plate.len_x) .* ...
            sin(plate.mode_index(:,2)*pi*accel.loc_y.'/ plate.len_y);


fprintf('===End of calculation of the resonance frequencies of the plate\n');


%% Calculate the coupling coefficients matrix (Bij)
% The coupling coefficient between a rectangular plate and a rectangular
% enclosure is [Bokil Eq. (60)]
%
% $$
%   C_{n,m} = \frac{(-1)^{n_3}}{\pi^2}
%                \times
%               \left[
%                   \frac{m_1}{n_1^2 - m_1^2}
%               \right]        
%                \times
%               \left[
%                   \frac{m_2}{n_2^2 - m_2^2}
%               \right]      
%               \times
%               \left[
%                   (-1)^{n_1 + m_1} - 1
%               \right]
%               \times
%               \left[
%                   (-1)^{n_2 + m_2} - 1
%               \right]
% $$
%
% where
% 
% * $n$ is the mode number of the cavity and has associated with it
% modal indicies $n_1, n_2, n_3$ along the $x,y,z$ axes.
% * $m$ is the mode number of the plate and has associated with it
% modal indicies $m_1, m_2$ along the $x,y$ axes.
%
% See also similar equations in papers by 
%
% * Jie Pan and David Alan Bies,
% "The effect of fluidstructural coupling on sound waves in an 
% enclosure - Theoretical part",
% Journal of the Acoustical Society of America,
% Volume 87, Issue 2, pp. 691-707 (1990)
% <http://dx.doi.org/10.1121/1.398939>
% * R.W. Guy, M.C. Bhattacharya,
%   "The transmission of sound through a cavity-backed finite plate",
%   Journal of Sound and Vibration,
%   Volume 27, Issue 2, 22 March 1973, Pages 207216, IN7IN8, p217223.
%   <http://dx.doi.org/10.1016/0022-460X(73)90062-X>
% * Sang-Myeong Kim and Michael J. Brennan,
% "Active control of harmonic sound transmission into an acoustic enclosure
% using both structural and acoustic actuators"
% Journal of the Acoustical Society of America,
% Volume 107, Issue 5, pp. 2523-2534 (2000).
% <http://dx.doi.org/10.1121/1.428640>
% * Scott Snyder and Nobuo Tanaka,
% "On feedforward active control of sound and vibration using
% vibration error signals", The Journal of the Acoustical Society of
% America, 94(4), October, 1993, p2181-2193.
% <http://dx.doi.org/10.1121/1.407489>


C_nm = zeros(ncavmodes,nstrmodes);
for nn_cav=1:ncavmodes
    for mm_plate=1:nstrmodes
%
% This was the equation from the paper in Bokil, 
        if (    ...
            (cavity.mode_index(nn_cav,1)~=plate.mode_index(mm_plate,1)) &     ...
            (cavity.mode_index(nn_cav,2)~=plate.mode_index(mm_plate,2)) ),      ...
                        
% For the system under investigation here, the flexible plate is mounted
% at $z=0$. The equation was derived using a symbolic toolbox.
% This expression is similar to :
% Guy and Bhattacharya, JSV, 1973, 27(2), p207-223. Eq(9)
% Note that the factor of 4 in their Eq(9) is not required here.
                C_nm(nn_cav,mm_plate) =                                  ...
                        (1./pi.^2)     ...
                        * (                            ...
                            plate.mode_index(mm_plate,1)     ...
                            .* plate.mode_index(mm_plate,2)       ...
                            )           ...
                        .*( (-1).^(cavity.mode_index(nn_cav,1) + plate.mode_index(mm_plate,1) ) -1 ) ...
                        .*( (-1).^(cavity.mode_index(nn_cav,2) + plate.mode_index(mm_plate,2) ) -1 ) ...
                        ./ (    ...
                            (cavity.mode_index(nn_cav,1).^2 - plate.mode_index(mm_plate,1).^2)     ...
                          .*(cavity.mode_index(nn_cav,2).^2 - plate.mode_index(mm_plate,2).^2) ...
                            );

%   This is the equation from papers by Bokil; Snyder et al, and others
%   where the flexible plate is at $z=L_z$.
%                 C_nm(nn_cav,mm_plate) =                                  ...
%                         (1./pi.^2)     ...
%                          * (-1).^(cavity.mode_index(nn_cav,3))     ...
%                         .* (                            ...
%                             plate.mode_index(mm_plate,1)     ...
%                             .* plate.mode_index(mm_plate,2)       ...
%                             )           ...
%                         .*( (-1).^(cavity.mode_index(nn_cav,1) + plate.mode_index(mm_plate,1) ) -1 ) ...
%                         .*( (-1).^(cavity.mode_index(nn_cav,2) + plate.mode_index(mm_plate,2) ) -1 ) ...
%                         ./ (    ...
%                             (cavity.mode_index(nn_cav,1).^2 - plate.mode_index(mm_plate,1).^2)     ...
%                           .*(cavity.mode_index(nn_cav,2).^2 - plate.mode_index(mm_plate,2).^2) ...
%                             );
                        
                        
        end;

    end;
end;     


% Pre-calculate some matrices before entering the frequency loop.
C_11_temp   = diag((plate.res_freq*2*pi).^2 .* plate.modalmass.');
C_12        = surfacearea*C_nm.';
C_21_temp   = surfacearea*C_nm;
C_22_temp1  = diag(cavity.modalvolume           ...
                .* (2*pi*cavity.res_freq).^2    ...
                / (densair*speedair.^2));
C_22_temp2  = diag(cavity.modalvolume           ...
                / (densair*speedair.^2));


    
%% Execute the big frequency loop
for ii=1:length(freq_vect),
    % disp(['Freq ',num2str(freq_vect(ii)),' of ',num2str(end_freq)]);
    omega = 2*pi*freq_vect(ii);

    load_vec =  [                       ...
                force_modal;              ...
                Q_speaker*(1j*omega)   ...
            ];

    
    LHS     =   [   C_11_temp-diag(plate.modalmass.')*(omega.^2)  C_12    ;   ...
                    (omega.^2)*C_21_temp                            C_22_temp1-omega.^2*C_22_temp2    ];
                
    small_q(:,ii) = inv(LHS)*load_vec;    
end; % of the big frequency loop
    
disp('End of calculations')

%% Separate the modal participation factors
% Separate them into the modal participation factors for the 
%
% * structure (top section), and 
% * acoustic response (bottom section).
%
struct_q=small_q(1:nstrmodes,:);
acoust_q=   small_q(nstrmodes+1:nstrmodes+ncavmodes,:);

        
%% Calculate some acoustic and structural results
% Use the modal participation factors to calculate some acoustic and
% structural results.
sq_struct_q=struct_q.^2;
sq_acoust_q=acoust_q.^2;

sum_sq_struct_q=sum(sq_struct_q);
sum_sq_acoust_q=sum(sq_acoust_q);

matlab_ape=1/(4*densair*speedair^2)*cavity.modalvolume.'*abs(acoust_q.^2);
matlab_pressure = cavity.phi_eval.' * acoust_q;
matlab_sq_p= matlab_pressure.^2;

matlab_displacement = plate.psi_eval.' * struct_q;

%% Plot the acoustic presure at the microphone
figure
set(gca,'fontsize',16);
p1h=semilogy(freq_vect,abs(matlab_pressure));
set(p1h,'linewidth',2);
xlabel('Frequency [Hz]');
ylabel('Pressure [Pa]');
title('Acoustic Pressure at a Point in the Cavity');
grid


%% Plot the displacement at the accelerometer
figure
set(gca,'fontsize',16);
p2h=semilogy(freq_vect,abs(matlab_displacement));
set(p2h,'linewidth',2);
xlabel('Frequency [Hz]');
ylabel('Displacement [m]');
title('Plate Displacement at Drive Point');
grid
